''' This module handles storing and retrieval of aliases from a file'''

import re
from . import config

ALIAS_DEFINITION_REGEX = r'\s*alias (\w+)=[\'\"](.+)[\'\"]'
ALIAS_SAVING_FORMAT = '\nalias {name}="{value}"'

def load_aliases():
    alias_pairs = []

    with open(config.FILE_PATH) as f:
        lines = f.readlines()

    for line in lines:
        m = re.match(ALIAS_DEFINITION_REGEX, line)
        if m:
            alias_pairs.append(m.groups())
    return alias_pairs

def save_aliases(alias_pairs):
    """ Write aliases to a file """
    lines = []
    with open(config.FILE_PATH, 'w') as f:
        for name, value in alias_pairs:
            lines.append(ALIAS_SAVING_FORMAT.format(name=name, value=value))
        f.writelines(lines)



''' Exceptions for alias manager'''

class Error(Exception):
    pass

class AliasExistsError(Error):
    message = 'Alias already exists'

class AliasNotFoundError(Error):
    message = 'Alias not found'

class InvalidAliasValueError(Error):
    message = 'Invalid alias value'



''' Alias manager for users of Unix shells'''

import sys
import re
import pkg_resources

from .manager import Manager
from .exceptions import AliasNotFoundError
from .config import HELP_TEXT, USAGE_TEXT, COLORS

manager = Manager()

# Valid command regex
ALIAS_NAME = r'([\w_-]+)'
ALIAS_VALUE = r'(.+)'
ADD = r'^'+ALIAS_NAME+'='+ALIAS_VALUE
DELETE = r'^rm '+ALIAS_NAME
LIST = r'^(list|ls)$'

def main(args_str):
	if args_str == '--help' or not args_str:
		default('{}{}'.format(HELP_TEXT, USAGE_TEXT))
		return

	if args_str == '--version':
		default(pkg_resources.get_distribution('aman').version)
		return

	for cmd, fn in functions.items():
		match = re.match(cmd, args_str)
		if match:
			fn(match)
			return

	error('Invalid command: {}'.format(args_str.split(' ')[0]))
	default(USAGE_TEXT)

def add_alias(match):
	name = match.group(1)
	value = match.group(2)

	try:
		alias = manager.get_alias(name)
	except AliasNotFoundError:
		manager.add(name, value)
		success('Alias added successfully.')
	else:
		action = prompt('Alias already exists. Edit? (y/n): ');
		if not action in 'yn':
			error('Invalid option: {}'.format(action))
		elif action == 'y':
			manager.edit(name, value)
			success('Alias updated.')

def delete_alias(match):
	name = match.group(1)
	manager.delete(name)

def list_aliases(*args):
	aliases = manager.get_all()

	for alias in aliases:
		success('{} {:<30} -> {}'.format(COLORS['SUCCESS'], alias.name, alias.value))

def prompt(message):
	if sys.version_info.major == 3:
		action = input(message)
	else:
		action = raw_input(message)
	return action

def default(message):
	print(message)

def success(message):
	print('{}{}\033[1;m'.format(COLORS['SUCCESS'], message))

def error(message):
	print('{}{}\033[1;m'.format(COLORS['ERROR'], message))

functions = {
	ADD: add_alias,
	DELETE: delete_alias,
	LIST: list_aliases
}
main(' '.join(sys.argv[1:]))


''' Aliases are managed here'''

from . import storage
from .alias import Alias

from .exceptions import (
    AliasNotFoundError,
    AliasExistsError,
    InvalidAliasValueError,
)

class Manager():
    def __init__(self):
        self.aliases = []
        for name, value in storage.load_aliases():
            self.aliases.append(Alias(name, value))

    def add(self, name, value):
        self.aliases.append(Alias(name, value))

        self.save()

    def edit(self, name, value):
        if not value:
            raise InvalidAliasError
        alias = self.get_alias(name)

        if alias:
            alias.value = value

        self.save()

    def modify(self, current_name, new_name):
        existing_alias = self.get_alias(new_name)
        if existing_alias:
            raise AliasExistsError

        alias = self.get_alias(current_name)
        alias.name = new_name

        self.save()

    def delete(self, name):
        alias = self.get_alias(name)
        if alias:
          self.aliases.remove(alias)

        self.save()

    def get_all(self):
        return self.aliases

    def get_alias(self, name):
        try:
            return list(filter(lambda a: a.name == name, self.aliases))[0]
        except IndexError:
            raise AliasNotFoundError

    def save(self):
        storage.save_aliases([(alias.name, alias.value) for alias in self.aliases])
        pass



import os

# Storage file settings
HOME_DIR = os.getenv('HOME')
FILE_NAME = 'aliases'
FILE_PATH = '{}/.aman/{}'.format(HOME_DIR, FILE_NAME)

# CLI settings
HELP_TEXT = 'Alias manager for shell lovers'
USAGE_TEXT = '''

USAGE:

  To add / edit an alias: 
  <name>="<value"

  list       List all aliases
  rm         Remove an alias
  --help     Show help
  --version  Show version number'''

COLORS = {
	'SUCCESS': '\033[0;32m',
	'ERROR': '\033[0;31m'
}



from distutils.core import setup
import sys
import os

setup(
    name='Aman',
    author='Hamza Ali',
    version='1.0',
    packages=['aman',],
    license='MIT',
    long_description=open('README.md').read(),
    scripts=['bin/aman']
)

# Creating directory containing aman storage file
HOME_PATH = os.getenv('HOME')
DIR = '.aman'
try:
	os.mkdir('{}/{}'.format(HOME_PATH, DIR))
except FileExistsError:
	pass

# Creating file to store aliases
ALIAS_FILE_NAME = 'aliases'
ALIAS_FILE_PATH = '{}/{}/{}'.format(HOME_PATH, DIR, ALIAS_FILE_NAME)
try:
	with open(ALIAS_FILE_PATH, 'w') as f:
		pass
except FileExistsError:
	pass
